//==========================================================
// Author: Lixin He,mohan
// DATE : 2008-11-6
//==========================================================
#include "../src_pw/global.h"
#include "../src_pw/tools.h"
#include "../input.h"
#include "update_input.h"
#include "../src_ions/ions_move_basic.h"
#include "../src_pw/optical.h"
#include "../src_lcao/bfield.h"
#include "../src_lcao/force_lcao.h"
#include "../src_lcao/local_orbital_charge.h"

Update_input::Update_input() {}
Update_input::~Update_input() {}

void Update_input::init(const string &fn)
{
	ofs_warning << "\n CHECK UPDATE INPUT INFORMATION" << endl;
    this->Read(fn);
#ifdef __MPI
    Bcast();
#endif

    return;
}

bool Update_input::Read(const string &fn)
{
    if (MY_RANK!=0) return false;

    ifstream ifs(fn.c_str(), ios::in);	// "in_datas/input_parameters"

    if (!ifs) 
	{
		return false;
	}

    ifs.clear();
    ifs.seekg(0);

    char word[80];
    char word1[80];
    int ierr = 0;

    //ifs >> setiosflags(ios::uppercase);
    ifs.rdstate();
    while (ifs.good())
    {
        ifs >> word;
        ifs.ignore(150, '\n');
        if (strcmp(word , "INPUT_PARAMETERS") == 0)
        {
            ierr = 1;
            break;
        }
        ifs.rdstate();
    }

    if (ierr == 0)
    {
		cout << " Error parameter list." << endl;
		return false;// return error : false
    }

    ifs.rdstate();

    while (ifs.good())
    {
        ifs >> word1;
        INPUT.strtolower(word1, word);
		
		// 1
        if (strcmp("force", word) == 0)
        {
            read_value(ifs, force);
			if(force!=FORCE)
			{
				this->change(ofs_warning,"FORCE",FORCE, force);
    			FORCE = this->force;
			}
        }
		// 2
        else if (strcmp("force_thr", word) == 0)
        {
            read_value(ifs, force_thr);
			if(force_thr!=FORCE_THR)
			{
				this->change(ofs_warning,"forth_thr(Ry/Bohr)",FORCE_THR,force_thr);
    			FORCE_THR = this->force_thr;
			}
        }
		// 2
        else if (strcmp("force_thr_ev", word) == 0)
        {
            read_value(ifs, force_thr);
			force_thr = force_thr / 13.6058 * 0.529177;
			if(force_thr!=FORCE_THR)
			{
				this->change(ofs_warning,"force_thr(Ry/Bohr)",FORCE_THR,force_thr);
    			FORCE_THR = this->force_thr;
			}
        }
		// 3
#ifdef __FP
        else if (strcmp("force_thr_ev2", word) == 0)
        {
            read_value(ifs, force_thr_ev2);
			if(force_thr_ev2!=Force_LCAO::force_invalid_threshold_ev)
			{
				this->change(ofs_warning,"force_thr threshold(Ry/Bohr)",Force_LCAO::force_invalid_threshold_ev,force_thr_ev2);
    			Force_LCAO::force_invalid_threshold_ev = this->force_thr_ev2;
			}
        }
#endif
		// 4
        else if (strcmp("dr2", word) == 0)
        {
            read_value(ifs, dr2);
			if(dr2!=DRHO2)
			{
				this->change(ofs_warning,"dr2",DRHO2,dr2);
    			DRHO2 = this->dr2;
			}
        }
		// 5
        else if (strcmp("niter", word) == 0)
        {
            read_value(ifs, niter);
			if(niter!=NITER)
			{
				this->change(ofs_warning,"niter",NITER,niter);
    			NITER = this->niter;
			}
        }
		// 6
        else if (strcmp("nstep", word) == 0)
        {
            read_value(ifs, nstep);
			if(nstep!=NSTEP)
			{
				this->change(ofs_warning,"nstep",NSTEP,nstep);
    			NSTEP = this->nstep;
			}
        }
		// 7
        else if (strcmp("mixing_beta", word) == 0)
        {
            read_value(ifs, mixing_beta);
			if(mixing_beta!=chr.mixing_beta)
			{
				this->change(ofs_warning,"mixing_beta",chr.mixing_beta,mixing_beta);
    			chr.mixing_beta = mixing_beta;
			}
        }
		// 8
        else if (strcmp("printe", word) == 0)
        {
            read_value(ifs, printe);
			if(printe!=en.printe)
			{
				this->change(ofs_warning,"printe",en.printe,printe);
				en.printe = this->printe; // mohan add 2011-03-16
			}
        }
		// 9 
        else if (strcmp("charge_extrap", word) == 0)//xiaohui modify 2015-02-01
        {
            read_value(ifs, charge_extrap);

			if(charge_extrap!=pot.extra_pot)
			{
				this->change(ofs_warning,"extra_pot",pot.extra_pot,charge_extrap);
				pot.extra_pot = this->charge_extrap;
				if(out_dm==0) 
				{
					out_dm = 10000;
#ifdef __FP
					this->change(ofs_warning,"out_dm",LOC.out_dm, out_dm);
					LOC.out_dm=out_dm;
#endif
				}
			}

        }
		// 10
        else if (strcmp("out_charge", word) == 0)
        {
            read_value(ifs, out_charge);
			if(out_charge!=chr.out_charge)
			{
				this->change(ofs_warning,"out_charge",chr.out_charge,out_charge);
    			chr.out_charge = this->out_charge;
			}
        }
		// 11
        else if (strcmp("out_dm", word) == 0)
        {
            read_value(ifs, out_dm);
#ifdef __FP
			if(out_dm!=LOC.out_dm)
			{
				this->change(ofs_warning,"out_dm",LOC.out_dm,out_dm);
				LOC.out_dm = this->out_dm;
			}
#endif
        }
		// 12
        else if (strcmp("out_dos", word) == 0)
        {
            read_value(ifs, out_dos);
			if(out_dos!=en.out_dos)
			{
				this->change(ofs_warning,"out_dos",en.out_dos,out_dos);
				en.out_dos = this->out_dos;
			}
        }
		// 13
        else if (strcmp("out_lowf", word) == 0)
        {
            read_value(ifs, out_lowf);
#ifdef __FP
			if(out_lowf!=out_lowf)
			{
				this->change(ofs_warning,"out_lowf",ParaO.out_lowf,out_lowf);
				ParaO.out_lowf = this->out_lowf;
			}
#endif
        }
        else
        {
            //cout << " THE PARAMETER NAME '" << word
            //     << "' IS NOT USED!" << endl;
            ifs.ignore(150, '\n');
        }

        ifs.rdstate();
        if (ifs.eof() != 0)
        {
			break;
        }
        else if (ifs.bad() != 0)
        {
			cout << " Bad input parameters. " << endl;
            return false;
        }
        else if (ifs.fail() != 0)
        {
			cout << " word = " << word << endl;
			cout << " Fail to read parameters. " << endl; 
            ifs.clear();
			return false;
        }
        else if (ifs.good() == 0)
        {
			break;
        }
    }

    return true;
}//end read_parameters

#ifdef __MPI
void Update_input::Bcast()
{
    Parallel_Common::bcast_int( FORCE );
    Parallel_Common::bcast_double( FORCE_THR);
    Parallel_Common::bcast_double( Force_LCAO::force_invalid_threshold_ev);
    Parallel_Common::bcast_double( DRHO2 );
    Parallel_Common::bcast_int( NITER );
    Parallel_Common::bcast_int( NSTEP );
    Parallel_Common::bcast_double( chr.mixing_beta );
    Parallel_Common::bcast_int( en.printe );
    Parallel_Common::bcast_string( pot.extra_pot );//xiaohui modify 2015-02-01
    Parallel_Common::bcast_int( chr.out_charge );
    Parallel_Common::bcast_int( LOC.out_dm );
	Parallel_Common::bcast_int( en.out_dos );
	Parallel_Common::bcast_int( ParaO.out_lowf );

    return;
}
#endif

